@@ -25,6 +25,7 @@
 
 #include "metkit/config/LibMetkit.h"
 
+#include "metkit/hypercube/HyperCube.h"
 #include "metkit/mars/MarsExpandContext.h"
 #include "metkit/mars/MarsExpension.h"
 #include "metkit/mars/MarsLanguage.h"
@@ -60,7 +61,6 @@ MarsLanguage::MarsLanguage(const std::string& verb) : verb_(verb) {
     eckit::Value lang   = languages_[verb];
     eckit::Value params = lang.keys();
 
-    eckit::Value defaults = lang["_defaults"];
     eckit::Value options  = lang["_options"];
 
     for (size_t i = 0; i < params.size(); ++i) {
@@ -73,11 +73,6 @@ MarsLanguage::MarsLanguage(const std::string& verb) : verb_(verb) {
 
         ASSERT(types_.find(keyword) == types_.end());
 
-
-        if (defaults.contains(keyword)) {
-            settings["default"] = defaults[keyword];
-        }
-
         if (options.contains(keyword)) {
             eckit::ValueMap m = options[keyword];
             for (auto j = m.begin(); j != m.end(); ++j) {
@@ -97,6 +92,22 @@ MarsLanguage::MarsLanguage(const std::string& verb) : verb_(verb) {
             }
         }
     }
+
+    std::set<std::string> keywordsInAxis;
+    for (const std::string& a : hypercube::AxisOrder::instance().axes()) {
+        keywordsInAxis.insert(a);
+        Type* t=nullptr;
+        auto it = types_.find(a);
+        if(it != types_.end()) {
+            t = (*it).second;
+        }
+        typesByAxisOrder_.push_back(std::make_pair(a,t)); 
+    }
+    for (const auto& [k,t] : types_) {
+        if (keywordsInAxis.find(k) == keywordsInAxis.end()) {
+            typesByAxisOrder_.push_back(std::make_pair(k,t));
+        }
+    }
 }
 
 MarsLanguage::~MarsLanguage() {
@@ -187,22 +198,8 @@ std::string MarsLanguage::bestMatch(const MarsExpandContext& ctx, const std::str
         std::cerr << "Matching '" << name << "' with " << best << ctx << std::endl;
     }
 
-    // size_t max = 3;
-    // if (best.size() > 0 && score < max) {
-    //     std::cerr << "Matching '"
-    //               << name
-    //               << "' with "
-    //               << best
-    //               << " "
-    //               << "Please give at least " << max << " first letters"
-    //               << std::endl;
-    // }
-
     if (best.size() == 1) {
         if (isnumeric(best[0]) && (best[0] != name)) {
-            // std::ostringstream oss;
-            // oss << "Invalid match [" << name << "] and [" << best[0] << "] (ignored)" << ctx;
-            // throw eckit::UserError(oss.str());
             best.clear();
         }
         else {
@@ -276,12 +273,6 @@ std::string MarsLanguage::bestMatch(const MarsExpandContext& ctx, const std::str
 
 std::string MarsLanguage::expandVerb(const MarsExpandContext& ctx, const std::string& verb) {
     pthread_once(&once, init);
-    // std::map<std::string, std::string>::iterator c = cache_.find(verb);
-    // if(c != cache_.end()) {
-    //     return (*c).second;
-    // }
-
-    // return cache_[verb] = bestMatch(verb, verbs_, true);
     return bestMatch(ctx, verb, verbs_, true, true, false);
 }
 
@@ -328,12 +319,6 @@ MarsRequest MarsLanguage::expand(const MarsExpandContext& ctx, const MarsRequest
                 p = cache_[*j] = bestMatch(ctx, *j, keywords_, true, false, false, aliases_);
             }
 
-            // if (seen.find(p) != seen.end()) {
-            //     std::cerr << "Duplicate " << p << " " << *j << std::endl;
-            // }
-
-            // seen.insert(p);
-
             std::vector<std::string> values = r.values(*j);
 
             if (values.size() == 1) {
@@ -348,23 +333,20 @@ MarsRequest MarsLanguage::expand(const MarsExpandContext& ctx, const MarsRequest
             type(p)->expand(ctx, values);
             result.setValuesTyped(type(p), values);
             type(p)->check(ctx, values);
-            // result.setValues(p, values);
         }
 
 
         if (inherit) {
-            for (std::map<std::string, Type*>::iterator k = types_.begin(); k != types_.end();
-                 ++k) {
-                const std::string& name = (*k).first;
-                if (result.countValues(name) == 0) {
-                    (*k).second->setDefaults(result);
+            for (const auto& [k,t] : typesByAxisOrder_) {
+                if (t != nullptr && result.countValues(k) == 0) {
+                    t->setDefaults(result);
                 }
             }
 
             result.getParams(params);
             for (std::vector<std::string>::const_iterator k = params.begin(); k != params.end();
                  ++k) {
-                type(*k)->setDefaults(result.values(*k));
+                type(*k)->setInheritance(result.values(*k));
             }
         }
 
